package io.gitlab.jfronny.muscript.ast.dynamic.assign;

import io.gitlab.jfronny.muscript.ast.*;
import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.error.LocationalException;

import java.io.IOException;

public class DynamicAssign extends DynamicExpr {
    private final String name;
    private final DynamicExpr value;

    public DynamicAssign(int chStart, int chEnd, String name, DynamicExpr value) {
        super(Order.Primary, chStart, chEnd);
        if (name.equals("this")) throw new LocationalException(chStart, chEnd, "Cannot reassign 'this'");
        this.name = name;
        this.value = value;
    }

    @Override
    public Dynamic<?> get(Scope dataRoot) {
        Dynamic<?> data = value.get(dataRoot);
        dataRoot.set(name, data);
        return data;
    }

    @Override
    public DynamicExpr optimize() {
        return new DynamicAssign(chStart, chEnd, name, value.optimize());
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        writer.appendLiteral(name).append(" = ");
        value.decompile(writer);
    }

    @Override
    public BoolExpr asBoolExpr() {
        return new BoolAssign(chStart, chEnd, name, value.asBoolExpr());
    }

    @Override
    public StringExpr asStringExpr() {
        return new StringAssign(chStart, chEnd, name, value.asStringExpr());
    }

    @Override
    public NumberExpr asNumberExpr() {
        return new NumberAssign(chStart, chEnd, name, value.asNumberExpr());
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof DynamicAssign assign
                && name.equals(assign.name)
                && value.equals(assign.value);
    }
}
